@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: pgsql2shp.c,v 1.83 2006/02/03 20:53:36 strk Exp $
+ * $Id: pgsql2shp.c 2513 2006-10-14 14:22:10Z mschaber $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -19,7 +19,7 @@
  **********************************************************************/
 
 static char rcsid[] =
-  "$Id: pgsql2shp.c,v 1.83 2006/02/03 20:53:36 strk Exp $";
+  "$Id: pgsql2shp.c 2513 2006-10-14 14:22:10Z mschaber $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,6 +34,8 @@ static char rcsid[] =
 #include "shapefil.h"
 #include "getopt.h"
 #include "compat.h"
+#include <sys/types.h> // for getpid()
+#include <unistd.h> // for getpid()
 
 #ifdef __CYGWIN__
 #include <sys/param.h>       
@@ -52,12 +54,17 @@ static char rcsid[] =
  * Verbosity:
  *   set to 1 to see record fetching progress
  *   set to 2 to see also shapefile creation progress
+ *   set to 3 for debugging
  */
 #define VERBOSE 1
 
 /* Define this to use HEX encoding instead of bytea encoding */
 #define HEXWKB 1
 
+/* Maximum DBF field width (according to ARCGIS) */
+#define MAX_DBF_FIELD_SIZE 255
+
+
 /*typedef unsigned long int uint32;*/
 typedef uint32_t uint32;
 typedef unsigned char byte;
@@ -65,6 +72,7 @@ typedef unsigned char byte;
 /* Global data */
 PGconn *conn;
 int rowbuflen;
+char temptablename[256];
 char *geo_col_name, *table, *shp_file, *schema, *usrquery;
 int type_ary[256];
 char *main_scan_query;
@@ -85,7 +93,7 @@ int pgis_major_version;
 /* Prototypes */
 int getMaxFieldSize(PGconn *conn, char *schema, char *table, char *fname);
 int parse_commandline(int ARGC, char **ARGV);
-void usage(int exitstatus);
+void usage(char* me, int exitstatus, FILE* out);
 char *getTableOID(char *schema, char *table);
 int addRecord(PGresult *res, int residx, int row);
 int initShapefile(char *shp_file, PGresult *res);
@@ -149,9 +157,10 @@ byte * HexDecode(const char *hex);
 #define ZMFLAG(x) (((x)&((WKBZOFFSET)+(WKBMOFFSET)))>>30)
 
 
-static void exit_nicely(PGconn *conn){
+static void exit_nicely(PGconn *conn, int code)
+{
 	PQfinish(conn);
-	exit(1);
+	exit(code);
 }
 
 int
@@ -185,10 +194,6 @@ main(int ARGC, char **ARGV)
 
 	if ( getenv("ROWBUFLEN") ) rowbuflen=atoi(getenv("ROWBUFLEN"));
 
-	if ( ARGC == 1 ) {
-		usage(0);
-	}
-
 	/*
 	 * Make sure dates are returned in ISO
 	 * style (YYYY-MM-DD).
@@ -200,7 +205,7 @@ main(int ARGC, char **ARGV)
 
 	if ( ! parse_commandline(ARGC, ARGV) ) {
                 printf("\n**ERROR** invalid option or command parameters\n\n");
-		usage(2);
+		usage(ARGV[0], 2, stderr);
 	}
 
 	/* Use table name as shapefile name */
@@ -210,7 +215,7 @@ main(int ARGC, char **ARGV)
 	conn = PQconnectdb("");
 	if (PQstatus(conn) == CONNECTION_BAD) {
 		printf( "%s", PQerrorMessage(conn));
-		exit_nicely(conn);
+		exit_nicely(conn, 1);
 	}
 
 	/* Create temporary table for user query */
@@ -228,7 +233,7 @@ main(int ARGC, char **ARGV)
 
 	/* Initialize shapefile and database infos */
 	fprintf(stdout, "Initializing... "); fflush(stdout);
-	if ( ! initialize() ) exit_nicely(conn);
+	if ( ! initialize() ) exit_nicely(conn, 1);
 	fprintf(stdout, "Done (postgis major version: %d).\n",
 		pgis_major_version); 
 
@@ -248,7 +253,7 @@ main(int ARGC, char **ARGV)
 	res=PQexec(conn, "BEGIN");
 	if ( ! res || PQresultStatus(res) != PGRES_COMMAND_OK ) {
 		printf( "%s", PQerrorMessage(conn));
-		exit_nicely(conn);
+		exit_nicely(conn, 1);
 	}
 	PQclear(res);
 
@@ -271,7 +276,7 @@ main(int ARGC, char **ARGV)
 	free(query);
 	if ( ! res || PQresultStatus(res) != PGRES_COMMAND_OK ) {
 		printf( "MainScanQuery: %s", PQerrorMessage(conn));
-		exit_nicely(conn);
+		exit_nicely(conn, 1);
 	}
 	PQclear(res);
 
@@ -295,7 +300,7 @@ main(int ARGC, char **ARGV)
 		if ( ! res || PQresultStatus(res) != PGRES_TUPLES_OK ) {
 			printf( "RecordFetch: %s",
 				PQerrorMessage(conn));
-			exit_nicely(conn);
+			exit_nicely(conn, 1);
 		}
 
 		/* No more rows, break the loop */
@@ -307,7 +312,7 @@ main(int ARGC, char **ARGV)
 		for(i=0; i<PQntuples(res); i++)
 		{
 			/* Add record in all output files */
-			if ( ! addRecord(res, i, row) ) exit_nicely(conn);
+			if ( ! addRecord(res, i, row) ) exit_nicely(conn, 1);
 			row++; 
 		}
 
@@ -323,13 +328,13 @@ main(int ARGC, char **ARGV)
 
 	if (dbf) DBFClose(dbf);
 	if (shp) SHPClose(shp);
-	exit_nicely(conn);
 
 
 #ifdef DEBUG
 	fclose(debug);
 #endif	 /* DEBUG */
 
+	exit_nicely(conn, 0);
 	return 0;
 }
 
@@ -2091,7 +2096,7 @@ getTableOID(char *schema, char *table)
 	free(query);
 	if ( ! res3 || PQresultStatus(res3) != PGRES_TUPLES_OK ) {
 		printf( "TableOID: %s", PQerrorMessage(conn));
-		exit_nicely(conn);
+		exit_nicely(conn, 1);
 	}
 	if(PQntuples(res3) == 1 ){
 		ret = strdup(PQgetvalue(res3, 0, 0));
@@ -2299,27 +2304,28 @@ getGeometryMaxDims(char *schema, char *table, char *geo_col_name)
 }
 
 void
-usage(int status)
+usage(char* me, int status, FILE* out)
 {
-	printf("RCSID: %s\n", rcsid);
-	printf("USAGE: pgsql2shp [<options>] <database> [<schema>.]<table>\n");
-	printf("       pgsql2shp [<options>] <database> <query>\n");
-	printf("\n");
-       	printf("OPTIONS:\n");
-       	printf("  -f <filename>  Use this option to specify the name of the file\n");
-       	printf("     to create.\n");
-       	printf("  -h <host>  Allows you to specify connection to a database on a\n");
-	printf("     machine other than the default.\n");
-       	printf("  -p <port>  Allows you to specify a database port other than the default.\n");
-       	printf("  -P <password>  Connect to the database with the specified password.\n");
-       	printf("  -u <user>  Connect to the database as the specified user.\n");
-	printf("  -g <geometry_column> Specify the geometry column to be exported.\n");
-	printf("  -b Use a binary cursor.\n");
-	printf("  -r Raw mode. Do not assume table has been created by \n");
-	printf("     the loader. This would not unescape attribute names\n");
-	printf("     and will not skip the 'gid' attribute.\n");
-	printf("  -k Keep postgresql identifiers case.\n");
-       	printf("\n");
+        fprintf(out,"RCSID: %s RELEASE: %s\n", rcsid, POSTGIS_VERSION);
+	fprintf(out,"USAGE: %s [<options>] <database> [<schema>.]<table>\n", me);
+	fprintf(out,"       %s [<options>] <database> <query>\n", me);
+	fprintf(out,"\n");
+       	fprintf(out,"OPTIONS:\n");
+       	fprintf(out,"  -f <filename>  Use this option to specify the name of the file\n");
+       	fprintf(out,"     to create.\n");
+       	fprintf(out,"  -h <host>  Allows you to specify connection to a database on a\n");
+	fprintf(out,"     machine other than the default.\n");
+       	fprintf(out,"  -p <port>  Allows you to specify a database port other than the default.\n");
+       	fprintf(out,"  -P <password>  Connect to the database with the specified password.\n");
+       	fprintf(out,"  -u <user>  Connect to the database as the specified user.\n");
+	fprintf(out,"  -g <geometry_column> Specify the geometry column to be exported.\n");
+	fprintf(out,"  -b Use a binary cursor.\n");
+	fprintf(out,"  -r Raw mode. Do not assume table has been created by \n");
+	fprintf(out,"     the loader. This would not unescape attribute names\n");
+	fprintf(out,"     and will not skip the 'gid' attribute.\n");
+	fprintf(out,"  -k Keep postgresql identifiers case.\n");
+        fprintf(out,"  -? Display this help screen.\n");
+       	fprintf(out,"\n");
        	exit (status);
 }
 
@@ -2327,6 +2333,10 @@ usage(int status)
 int
 parse_commandline(int ARGC, char **ARGV)
 {
+        if ( ARGC == 1 ) {
+                usage(ARGV[0], 0, stdout);
+        }
+
 	int c, curindex;
 	char buf[1024];
 
@@ -2376,6 +2386,7 @@ parse_commandline(int ARGC, char **ARGV)
 				keep_fieldname_case = 1;
 				break;
 			case '?':
+                                usage(ARGV[0], 0, stdout);
 			default:
 				return 0;
 		}
@@ -2500,7 +2511,6 @@ initialize(void)
 		return 0;
 	}
 
-
 	/* Get geometry oid */
 	geo_oid = getGeometryOID(conn);
 	if ( geo_oid == -1 )
@@ -2772,12 +2782,22 @@ initialize(void)
 				/* might 0 be a good size ? */
 			}
 		}
+
+		if ( size > MAX_DBF_FIELD_SIZE )
+		{
+			fprintf(stderr, "Warning: values of field '%s' "
+				"exceeding maximum dbf field width (%d) "
+				"will be truncated.\n",
+				fname, MAX_DBF_FIELD_SIZE);
+			size = MAX_DBF_FIELD_SIZE;
+		}
+
 /*printf( "FIELD_NAME: %s, SIZE: %d\n", field_name, size); */
-		
+
 		/* generic type (use string representation) */
 		if(DBFAddField(dbf, field_name, FTString, size, 0) == -1)
 		{
-			printf( "Error - String field could not "
+			fprintf(stderr, "Error - String field could not "
 					"be created.\n");
 			return 0;
 		}
@@ -3225,7 +3245,15 @@ parse_table(char *spec)
 	if ( strstr(spec, "SELECT ") || strstr(spec, "select ") )
 	{
 		usrquery = spec;
-		table = "__pgsql2shp_tmp_table";
+
+		/*
+		 * encode pid in table name to reduce 
+		 * clashes probability (see bug#115)
+		 */
+		sprintf(temptablename,
+			"__pgsql2shp%lu_tmp_table",
+			(long)getpid());
+		table = temptablename; 
 	}
 	else
 	{
@@ -3326,7 +3354,14 @@ goodDBFValue(const char *in, char fieldType)
 }
 
 /**********************************************************************
- * $Log: pgsql2shp.c,v $
+ * $Log$
+ * Revision 1.85  2006/06/16 14:12:16  strk
+ *         - BUGFIX in pgsql2shp successful return code.
+ *         - BUGFIX in shp2pgsql handling of MultiLine WKT.
+ *
+ * Revision 1.84  2006/04/18 14:09:28  strk
+ * Limited text field size to 255 (bug #84)  [will eventually provide a switch to support wider fields ]
+ *
  * Revision 1.83  2006/02/03 20:53:36  strk
  * Swapped stdint.h (unavailable on Solaris9) with inttypes.h
  *
