@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: shp2pgsql.c,v 1.106 2006/01/16 10:42:58 strk Exp $
+ * $Id: shp2pgsql.c 2513 2006-10-14 14:22:10Z mschaber $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -69,6 +69,7 @@ enum {
 
 /* globals */
 int	dump_format = 0; /* 0=insert statements, 1 = dump */
+int     simple_geometries = 0; /* 0 = MULTILINESTRING/MULTIPOLYGON, 1 = LINESTRING/POLYGON */
 int	quoteidentifiers = 0;
 int	forceint4 = 0;
 int	createindex = 0;
@@ -105,7 +106,7 @@ int PIP( Point P, Point* V, int n );
 void *safe_malloc(size_t size);
 void CreateTable(void);
 void CreateIndex(void);
-void usage(char *me, int exitcode);
+void usage(char *me, int exitcode, FILE* out);
 void InsertPoint(void);
 void InsertPointWKT(void);
 void InsertMultiPoint(void);
@@ -137,7 +138,7 @@ static void print_wkb_int(int val);
 static void print_wkb_double(double val);
 
 static char rcsid[] =
-  "$Id: shp2pgsql.c,v 1.106 2006/01/16 10:42:58 strk Exp $";
+  "$Id: shp2pgsql.c 2513 2006-10-14 14:22:10Z mschaber $";
 
 void *safe_malloc(size_t size)
 {
@@ -380,7 +381,7 @@ main (int ARGC, char **ARGV)
 	/*
 	 * Parse command line
 	 */
-	if ( ! ParseCmdline(ARGC, ARGV) ) usage(ARGV[0], 2);
+	if ( ! ParseCmdline(ARGC, ARGV) ) usage(ARGV[0], 2, stderr);
 
 	/*
 	 * Open shapefile and initialize globals
@@ -730,43 +731,34 @@ LoadData(void)
 }
 
 void
-usage(char *me, int exitcode)
+usage(char *me, int exitcode, FILE* out)
 {
-	fprintf(stderr, "RCSID: %s\n", rcsid);
-	fprintf(stderr, "USAGE: %s [<options>] <shapefile> [<schema>.]<table>\n", me);
-	fprintf(stderr, "\n");
-	fprintf(stderr, "OPTIONS:\n");
-	fprintf(stderr, "  -s <srid>  Set the SRID field. If not specified it defaults to -1.\n");
-	fprintf(stderr, "\n");
-	fprintf(stderr, "  (-d|a|c|p) These are mutually exclusive options:\n");
-	fprintf(stderr, "      -d  Drops the table , then recreates it and populates\n");
-	fprintf(stderr, "          it with current shape file data.\n");
-	fprintf(stderr, "      -a  Appends shape file into current table, must be\n");
-	fprintf(stderr, "          exactly the same table schema.\n");
-	fprintf(stderr, "      -c  Creates a new table and populates it, this is the\n");
-	fprintf(stderr, "          default if you do not specify any options.\n");
- 	fprintf(stderr, "      -p  Prepare mode, only creates the table\n");
-	fprintf(stderr, "\n");
-	fprintf(stderr, "  -g <geometry_column> Specify the name of the geometry column\n");
-	fprintf(stderr, "     (mostly useful in append mode).\n");
-	fprintf(stderr, "\n");
-	fprintf(stderr, "  -D  Use postgresql dump format (defaults to sql insert\n");
-	fprintf(stderr, "      statments.\n");
-	fprintf(stderr, "\n");
-	fprintf(stderr, "  -k  Keep postgresql identifiers case.\n");
-	fprintf(stderr, "\n");
-	fprintf(stderr, "  -i  Use int4 type for all integer dbf fields.\n");
-	fprintf(stderr, "\n");
-	fprintf(stderr, "  -I  Create a GiST index on the geometry column.\n");
-	fprintf(stderr, "\n");
-	fprintf(stderr, "  -w  Use wkt format (for postgis-0.x support - drops M - drifts coordinates).\n");
+        fprintf(out, "RCSID: %s RELEASE: %s\n", rcsid, POSTGIS_VERSION);
+	fprintf(out, "USAGE: %s [<options>] <shapefile> [<schema>.]<table>\n", me);
+	fprintf(out, "OPTIONS:\n");
+	fprintf(out, "  -s <srid>  Set the SRID field. If not specified it defaults to -1.\n");
+	fprintf(out, "  (-d|a|c|p) These are mutually exclusive options:\n");
+	fprintf(out, "      -d  Drops the table, then recreates it and populates\n");
+	fprintf(out, "          it with current shape file data.\n");
+	fprintf(out, "      -a  Appends shape file into current table, must be\n");
+	fprintf(out, "          exactly the same table schema.\n");
+	fprintf(out, "      -c  Creates a new table and populates it, this is the\n");
+	fprintf(out, "          default if you do not specify any options.\n");
+ 	fprintf(out, "      -p  Prepare mode, only creates the table.\n");
+	fprintf(out, "  -g <geometry_column> Specify the name of the geometry column\n");
+	fprintf(out, "     (mostly useful in append mode).\n");
+	fprintf(out, "  -D  Use postgresql dump format (defaults to sql insert statments.\n");
+	fprintf(out, "  -k  Keep postgresql identifiers case.\n");
+	fprintf(out, "  -i  Use int4 type for all integer dbf fields.\n");
+	fprintf(out, "  -I  Create a GiST index on the geometry column.\n");
+	fprintf(out, "  -S  Generate simple geometries instead of MULTI geometries.\n");
+	fprintf(out, "  -w  Use wkt format (for postgis-0.x support - drops M - drifts coordinates).\n");
 #ifdef USE_ICONV
-	fprintf(stderr, "\n");
-	fprintf(stderr, "  -W <encoding> Specify the character encoding of Shape's\n");
-	fprintf(stderr, "     attribute column. (default : \"ASCII\")\n");
+	fprintf(out, "  -W <encoding> Specify the character encoding of Shape's\n");
+	fprintf(out, "     attribute column. (default : \"ASCII\")\n");
 #endif
-	fprintf(stderr, "\n");
-	fprintf(stderr, "  -N <policy> Specify NULL geometries handling policy (insert,skip,abort)\n");
+	fprintf(out, "  -N <policy> Specify NULL geometries handling policy (insert,skip,abort)\n");
+        fprintf(out, "  -? Display this help screen\n");
 	exit (exitcode);
 }
 
@@ -792,9 +784,17 @@ InsertLineString(int id)
 	if (!dump_format) printf("'");
 	if ( sr_id && sr_id != "-1" ) printf("SRID=%s;", sr_id);
 
-	print_wkb_byte(getEndianByte());
-	print_wkb_int(wkbtype);
-	print_wkb_int(obj->nParts);
+	if (simple_geometries==0) // We write MULTI geometries, so generate Header 
+	{
+		print_wkb_byte(getEndianByte());
+		print_wkb_int(wkbtype);
+		print_wkb_int(obj->nParts); /* npolys */
+	} 
+	else if ((obj->nParts)!=1) // We write Non-MULTI geometries, but have several parts: 
+	{
+		fprintf(stderr, "We have a MultiLineString with %d parts, can't use -S switch!\n", obj->nParts);
+		exit(1);		
+	}
 
 	for (pi=0; pi<obj->nParts; pi++)
 	{
@@ -843,8 +843,22 @@ InsertLineStringWKT(int id)
 		return;
 	}
 
-	if (dump_format) printf("SRID=%s;MULTILINESTRING(",sr_id);
-	else printf("GeometryFromText('MULTILINESTRING (");
+	if (dump_format) printf("SRID=%s;",sr_id );
+	else printf("GeometryFromText('");
+	
+	if (simple_geometries==0) // We write MULTI geometries, so generate Header 
+	{
+		printf("MULTILINESTRING(");
+	}
+	else if ((obj->nParts)==1)
+	{
+		printf("LINESTRING");
+	}
+	else // We write Non-MULTI geometries, but have several parts: 
+	{
+		fprintf(stderr, "We have a Multilinestring with %d parts, can't use -S switch!\n", obj->nParts);
+		exit(1);		
+	} 
 
 	for (pi=0; pi<obj->nParts; pi++)
 	{
@@ -862,7 +876,7 @@ InsertLineStringWKT(int id)
 
 		for ( vi=vs; vi<ve; vi++)
 		{
-			if ( vi ) printf(",");
+			if ( vi > vs ) printf(",");
 			printf("%.15g %.15g",
 				obj->padfX[vi],
 				obj->padfY[vi]);
@@ -874,8 +888,10 @@ InsertLineStringWKT(int id)
 				
 	}
 
-	if (dump_format) printf(")\n");
-	else printf(")',%s) );\n",sr_id);
+	if (simple_geometries==0) printf(")");
+
+	if (dump_format) printf("\n");
+	else printf("',%s) );\n",sr_id);
 }
 
 int
@@ -1051,9 +1067,17 @@ InsertPolygon(void)
 	if (!dump_format) printf("'");
 	if ( sr_id && sr_id != "-1" ) printf("SRID=%s;", sr_id);
 
-	print_wkb_byte(getEndianByte());
-	print_wkb_int(wkbtype);
-	print_wkb_int(out_index); /* npolys */
+	if (simple_geometries==0) // We write MULTI geometries, so generate Header 
+	{
+		print_wkb_byte(getEndianByte());
+		print_wkb_int(wkbtype);
+		print_wkb_int(out_index); /* npolys */
+	} 
+	else if (out_index!=1) // We write Non-MULTI geometries, but have several parts: 
+	{
+		fprintf(stderr, "We have a Multipolygon with %d parts, can't use -S switch!\n", out_index);
+		exit(1);		
+	}
 
 	/* Write the coordinates */
 	for(pi=0; pi<out_index; pi++)
@@ -1112,8 +1136,22 @@ InsertPolygonWKT(void)
 
 	out_index = FindPolygons(obj, &Outer);
 
-	if (dump_format) printf("SRID=%s;MULTIPOLYGON(",sr_id );
-	else printf("GeometryFromText('MULTIPOLYGON(");
+	if (dump_format) printf("SRID=%s;",sr_id );
+	else printf("GeometryFromText('");
+	
+	if (simple_geometries==0) // We write MULTI geometries, so generate Header 
+	{
+		printf("MULTIPOLYGON(");
+	} 
+	else if (out_index==1) 
+	{
+		printf("POLYGON");
+	}
+	else
+	{ // We write Non-MULTI geometries, but have several parts: 
+		fprintf(stderr, "We have a Multipolygon with %d parts, can't use -S switch!\n", out_index);
+		exit(1);		
+	}
 
 	/* Write the coordinates */
 	for(pi=0; pi<out_index; pi++)
@@ -1149,8 +1187,10 @@ InsertPolygonWKT(void)
 
 	}
 
-	if (dump_format) printf(")\n");
-	else printf(")',%s) );\n",sr_id);
+	if (simple_geometries==0) printf(")");
+	
+	if (dump_format) printf("\n");
+	else printf("',%s) );\n",sr_id);
 
 	/* Release all memory */
 	ReleasePolygons(Outer, out_index);
@@ -1228,7 +1268,11 @@ ParseCmdline(int ARGC, char **ARGV)
 	extern char *optarg;
 	extern int optind;
 
-	while ((c = getopt(ARGC, ARGV, "kcdapDs:g:iW:wIN:")) != EOF){
+        if ( ARGC == 1 ) {
+                usage(ARGV[0], 0, stdout);
+        }
+
+	while ((c = getopt(ARGC, ARGV, "kcdapDs:Sg:iW:wIN:")) != EOF){
                switch (c) {
                case 'c':
                     if (opt == ' ')
@@ -1257,6 +1301,9 @@ ParseCmdline(int ARGC, char **ARGV)
 	       case 'D':
 		    dump_format =1;
                     break;
+               case 'S':
+                    simple_geometries =1;
+                    break;
                case 's':
                     sr_id = optarg;
                     break;
@@ -1300,6 +1347,7 @@ ParseCmdline(int ARGC, char **ARGV)
 			}
                     break;
                case '?':
+                    usage(ARGV[0], 0, stdout); 
                default:              
 		return 0;
                }
@@ -1311,7 +1359,7 @@ ParseCmdline(int ARGC, char **ARGV)
 
 	if ( opt==' ' ) opt = 'c';
 
-        for (; optind < ARGC; optind++){
+	for (; optind < ARGC; optind++){
 		if(curindex ==0){
 			shp_file = ARGV[optind];
 		}else if(curindex == 1){
@@ -1449,6 +1497,13 @@ SetPgType(void)
 				shpfiletype);
 			break;
 	}
+
+        if (simple_geometries)
+        {
+                // adjust geometry name for CREATE TABLE by skipping MULTI
+                if ((wkbtype & 0x7) == MULTIPOLYGONTYPE) pgtype += 5;
+                if ((wkbtype & 0x7) == MULTILINETYPE) pgtype += 5;
+        }                        
 }
 
 char *
@@ -1680,7 +1735,7 @@ utf8 (const char *fromcode, char *inputbuf)
 		fprintf(stderr, "utf8: malloc: %s\n", strerror (errno));
 		return NULL;
 	}
-	bzero (outputbuf, outbytesleft);
+	memset (outputbuf, 0, outbytesleft);
 	outputptr = outputbuf;
 
 	if (-1==iconv(cd, &inputbuf, &inbytesleft, &outputptr, &outbytesleft))
@@ -1697,7 +1752,14 @@ utf8 (const char *fromcode, char *inputbuf)
 #endif /* defined USE_ICONV */
 
 /**********************************************************************
- * $Log: shp2pgsql.c,v $
+ * $Log$
+ * Revision 1.108  2006/06/16 14:12:17  strk
+ *         - BUGFIX in pgsql2shp successful return code.
+ *         - BUGFIX in shp2pgsql handling of MultiLine WKT.
+ *
+ * Revision 1.107  2006/04/18 09:16:26  strk
+ * Substituted bzero() use with memset()
+ *
  * Revision 1.106  2006/01/16 10:42:58  strk
  * Added support for Bool and Date DBF<=>PGIS mapping
  *
