@@ -1,3 +1,15 @@
+/**********************************************************************
+ * $Id: lwgeom_functions_basic.c 2461 2006-07-28 13:07:06Z strk $
+ *
+ * PostGIS - Spatial Types for PostgreSQL
+ * http://postgis.refractions.net
+ * Copyright 2001-2006 Refractions Research Inc.
+ *
+ * This is free software; you can redistribute and/or modify it under
+ * the terms of the GNU General Public Licence. See the COPYING file.
+ * 
+ **********************************************************************/
+
 #include <math.h>
 #include <float.h>
 #include <string.h>
@@ -1610,6 +1622,7 @@ Datum LWGEOM_collect(PG_FUNCTION_ARGS)
 	LWGEOM *lwgeoms[2], *outlwg;
 	unsigned int type1, type2, outtype;
 	BOX2DFLOAT4 *box=NULL;
+	int SRID;
 
 	/* return null if both geoms are null */
 	if ( (geom1_ptr == NULL) && (geom2_ptr == NULL) )
@@ -1639,11 +1652,16 @@ Datum LWGEOM_collect(PG_FUNCTION_ARGS)
 	elog(NOTICE, "LWGEOM_collect(%s, %s): call", lwgeom_typename(TYPE_GETTYPE(pglwgeom1->type)), lwgeom_typename(TYPE_GETTYPE(pglwgeom2->type)));
 #endif
 	
+#if 0
 	if ( pglwgeom_getSRID(pglwgeom1) != pglwgeom_getSRID(pglwgeom2) )
 	{
 		elog(ERROR, "Operation on two GEOMETRIES with different SRIDs\n");
 		PG_RETURN_NULL();
 	}
+#endif
+
+	SRID = pglwgeom_getSRID(pglwgeom1);
+	errorIfSRIDMismatch(SRID, pglwgeom_getSRID(pglwgeom2));
 
 	lwgeoms[0] = lwgeom_deserialize(SERIALIZED_FORM(pglwgeom1));
 	lwgeoms[1] = lwgeom_deserialize(SERIALIZED_FORM(pglwgeom2));
@@ -1674,7 +1692,7 @@ Datum LWGEOM_collect(PG_FUNCTION_ARGS)
 	lwgeom_dropSRID(lwgeoms[1]);
 
 	outlwg = (LWGEOM *)lwcollection_construct(
-		outtype, lwgeoms[0]->SRID,
+		outtype, SRID,
 		box, 2, lwgeoms);
 
 	result = pglwgeom_serialize(outlwg);
@@ -1748,7 +1766,7 @@ Datum LWGEOM_accum(PG_FUNCTION_ARGS)
 	/* Make a DETOASTED copy of input geometry */
 	geom = (PG_LWGEOM *)PG_DETOAST_DATUM(datum);
 #ifdef PGIS_DEBUG
-	elog(NOTICE, "geom_accum: detoasted geom: %x", geom);
+	elog(NOTICE, "geom_accum: detoasted geom: %p", (void*)geom);
 #endif
 
 	/*
@@ -1761,7 +1779,7 @@ Datum LWGEOM_accum(PG_FUNCTION_ARGS)
 		nbytes = ARR_OVERHEAD_NONULLS(1)+INTALIGN(geom->size);
 #ifdef PGIS_DEBUG
 		elog(NOTICE, "geom_accum: adding %p (nelems=%d; nbytes=%d)",
-			geom, nelems, nbytes);
+			(void*)geom, nelems, nbytes);
 #endif
 		result = lwalloc(nbytes);
 		if ( ! result )
@@ -1790,7 +1808,7 @@ Datum LWGEOM_accum(PG_FUNCTION_ARGS)
 		oldsize = array->size;
 		nbytes = oldsize + INTALIGN(geom->size);
 #ifdef PGIS_DEBUG
-		elog(NOTICE, "geom_accum: old array size: %d, adding %d bytes (nelems=%d; nbytes=%lu)", array->size, INTALIGN(geom->size), nelems, nbytes);
+		elog(NOTICE, "geom_accum: old array size: %d, adding %ld bytes (nelems=%d; nbytes=%u)", array->size, INTALIGN(geom->size), nelems, nbytes);
 #endif
 		result = (ArrayType *) lwrealloc(array, nbytes);
 		if ( ! result )
@@ -1803,7 +1821,7 @@ Datum LWGEOM_accum(PG_FUNCTION_ARGS)
 #endif
 
 #ifdef PGIS_DEBUG
-		elog(NOTICE, " array start  @ %p", result);
+		elog(NOTICE, " array start  @ %p", (void*)result);
 		elog(NOTICE, " ARR_DATA_PTR @ %p (%d)",
 			ARR_DATA_PTR(result), (uchar *)ARR_DATA_PTR(result)-(uchar *)result);
 		elog(NOTICE, " next element @ %p", (uchar *)result+oldsize);
@@ -1812,7 +1830,7 @@ Datum LWGEOM_accum(PG_FUNCTION_ARGS)
 		memcpy(ARR_DIMS(result), &nelems, sizeof(int));
 #ifdef PGIS_DEBUG
 		elog(NOTICE, " writing next element starting @ %p",
-			result+oldsize);
+			(void*)(result+oldsize));
 #endif
 		memcpy((uchar *)result+oldsize, geom, geom->size);
 	}
@@ -1867,7 +1885,7 @@ Datum LWGEOM_collect_garray(PG_FUNCTION_ARGS)
 	array = (ArrayType *) PG_DETOAST_DATUM(datum);
 
 #ifdef PGIS_DEBUG
-	elog(NOTICE, " array is %d-bytes in size, %d w/out header",
+	elog(NOTICE, " array is %d-bytes in size, %ld w/out header",
 		array->size, array->size-ARR_OVERHEAD_NONULLS(ARR_NDIM(array)));
 #endif
 
@@ -1989,7 +2007,7 @@ Datum LWGEOM_line_from_mpoint(PG_FUNCTION_ARGS)
 	LWMPOINT *mpoint;
 
 #ifdef PGIS_DEBUG
-	elog(NOTICE, "LWGEOM_makeline called");
+	elog(NOTICE, "LWGEOM_line_from_mpoint called");
 #endif
 
 	/* Get input PG_LWGEOM and deserialize it */
@@ -2156,11 +2174,7 @@ Datum LWGEOM_makeline(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 	}
 
-	if ( pglwgeom_getSRID(pglwg1) != pglwgeom_getSRID(pglwg2) )
-	{
-		elog(ERROR, "Operation with two geometries with different SRIDs\n");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(pglwgeom_getSRID(pglwg1), pglwgeom_getSRID(pglwg2));
 
 	lwpoints[0] = lwpoint_deserialize(SERIALIZED_FORM(pglwg1));
 	lwpoints[1] = lwpoint_deserialize(SERIALIZED_FORM(pglwg2));
@@ -2226,7 +2240,7 @@ Datum LWGEOM_makepoly(PG_FUNCTION_ARGS)
 
 	outpoly = lwpoly_from_lwlines(shell, nholes, holes);
 #ifdef PGIS_DEBUG
-	lwnotice("%s", lwpoly_summary(outpoly));
+	lwnotice("%s", lwgeom_summary((LWGEOM*)outpoly, 0));
 #endif
 
 	result = pglwgeom_serialize((LWGEOM *)outpoly);
@@ -2381,7 +2395,7 @@ Datum LWGEOM_isempty(PG_FUNCTION_ARGS)
 
 
 /*
- *  Returns a modified [multi]polygon so that no ring segment is 
+ *  Returns a modified geometry so that no segment is 
  *  longer then the given distance (computed using 2d).
  *  Every input point is kept.
  *  Z and M values for added points (if needed) are set to 0.
@@ -2957,8 +2971,15 @@ lwgeom_affine_ptarray(POINTARRAY *pa,
 	double x,y,z;
 	POINT4D p4d;
 
+#ifdef PGIS_DEBUG
+	lwnotice("lwgeom_affine_ptarray start");
+#endif
+
 	if ( TYPE_HASZ(pa->dims) )
 	{
+#ifdef PGIS_DEBUG
+		lwnotice(" has z");
+#endif
 		for (i=0; i<pa->npoints; i++) {
 			getPoint4d_p(pa, i, &p4d);
 			x = p4d.x;
@@ -2968,10 +2989,16 @@ lwgeom_affine_ptarray(POINTARRAY *pa,
 			p4d.y = dfac * x + efac * y + ffac * z + yoff;
 			p4d.z = gfac * x + hfac * y + ifac * z + zoff;
 			setPoint4d(pa, i, &p4d);
+#ifdef PGIS_DEBUG
+		lwnotice(" POINT %g %g %g => %g %g %g", x, y, x, p4d.x, p4d.y, p4d.z);
+#endif
 		}
 	}
 	else
 	{
+#ifdef PGIS_DEBUG
+		lwnotice(" doesn't have z");
+#endif
 		for (i=0; i<pa->npoints; i++) {
 			getPoint4d_p(pa, i, &p4d);
   			x = p4d.x;
@@ -2979,8 +3006,16 @@ lwgeom_affine_ptarray(POINTARRAY *pa,
   			p4d.x = afac * x + bfac * y + xoff;
   			p4d.y = dfac * x + efac * y + yoff;
 			setPoint4d(pa, i, &p4d);
+#ifdef PGIS_DEBUG
+		lwnotice(" POINT %g %g %g => %g %g %g", x, y, x, p4d.x, p4d.y, p4d.z);
+#endif
 		}
 	}
+
+#ifdef PGIS_DEBUG
+	lwnotice("lwgeom_affine_ptarray end");
+#endif
+
 }
 
 void
@@ -3088,6 +3123,8 @@ Datum LWGEOM_affine(PG_FUNCTION_ARGS)
 
 	/* COMPUTE_BBOX TAINTING */
 	tmp = pglwgeom_deserialize(geom);
+	lwgeom_dropBBOX(tmp);
+	tmp->bbox = lwgeom_compute_box2d(tmp);
 	ret = pglwgeom_serialize(tmp);
 
 	/* Release memory */
