@@ -9,6 +9,33 @@
 #include "profile.h"
 #include "wktparse.h"
 
+#define PARANOIA_LEVEL 1
+
+/*
+ * Define this to have have many notices printed
+ * during postgis->geos and geos->postgis conversions
+ */
+//#define PGIS_DEBUG_CONVERTER 1
+#ifdef PGIS_DEBUG_CONVERTER
+#define PGIS_DEBUG_POSTGIS2GEOS 1
+#define PGIS_DEBUG_GEOS2POSTGIS 1
+#endif // PGIS_DEBUG_CONVERTER
+//#define PGIS_DEBUG 1
+
+
+//
+// WARNING: buffer-based GeomUnion has been disabled due to
+//          limitations in the GEOS code (it would only work
+//	    against polygons)
+//
+// Fuzzy way of finding out how many points to stuff
+// in each chunk: 680 * Mb of memory
+//
+// The example below is for about 32 MB (fuzzy pragmatic check)
+//
+//#define UNITE_USING_BUFFER 1
+#define MAXGEOMSPOINTS 21760
+
 Datum relate_full(PG_FUNCTION_ARGS);
 Datum relate_pattern(PG_FUNCTION_ARGS);
 Datum disjoint(PG_FUNCTION_ARGS);
@@ -33,22 +60,13 @@ Datum geomequals(PG_FUNCTION_ARGS);
 Datum pointonsurface(PG_FUNCTION_ARGS);
 Datum GEOSnoop(PG_FUNCTION_ARGS);
 Datum postgis_geos_version(PG_FUNCTION_ARGS);
+Datum postgis_jts_version(PG_FUNCTION_ARGS);
 Datum centroid(PG_FUNCTION_ARGS);
-Datum GEOS_polygonize_garray(PG_FUNCTION_ARGS);
+Datum polygonize_garray(PG_FUNCTION_ARGS);
+Datum linemerge(PG_FUNCTION_ARGS);
+Datum LWGEOM_buildarea(PG_FUNCTION_ARGS);
 
 
-
-/*
- * Define this to have have many notices printed
- * during postgis->geos and geos->postgis conversions
- */
-//#define PGIS_DEBUG_CONVERTER 1
-#ifdef PGIS_DEBUG_CONVERTER
-#define PGIS_DEBUG_POSTGIS2GEOS 1
-#define PGIS_DEBUG_GEOS2POSTGIS 1
-#endif // PGIS_DEBUG_CONVERTER
-//#define PGIS_DEBUG 1
-
 typedef  struct Geometry Geometry;
 
 extern const char * createGEOSPoint(POINT3D *pt);
@@ -94,6 +112,7 @@ extern Geometry	*GEOSGetGeometryN(Geometry *g1, int n);
 extern Geometry	*GEOSGetExteriorRing(Geometry *g1);
 extern Geometry	*GEOSGetInteriorRingN(Geometry *g1,int n);
 extern Geometry *GEOSpolygonize(Geometry **geoms, unsigned int ngeoms);
+extern Geometry *GEOSLineMerge(Geometry *geoms);
 extern int	GEOSGetNumInteriorRings(Geometry *g1);
 extern int      GEOSGetSRID(Geometry *g1);
 extern int      GEOSGetNumGeometries(Geometry *g1);
@@ -105,8 +124,18 @@ extern Geometry *PostGIS2GEOS_multipolygon(LWPOLY **geoms, uint32 ngeoms, int SR
 extern Geometry *PostGIS2GEOS_multilinestring(LWLINE **geoms, uint32 ngeoms, int SRID, int is3d);
 extern Geometry *PostGIS2GEOS_multipoint(LWPOINT **geoms, uint32 ngeoms, int SRID, int is3d);
 extern Geometry *PostGIS2GEOS_collection(int type, Geometry **geoms, int ngeoms, int SRID, bool is3d);
+/*
+ * Returned Polygon will takes ownership of all args
+ * Taken from GEOS CAPI
+ */
+extern Geometry *GEOSCreatePolygon(Geometry *shell, Geometry **holes, 
+	unsigned int nholes);
+/*
+ * Clone a geometry
+ */
+extern Geometry *GEOSCloneGeometry(const Geometry *g);
 
-void NOTICE_MESSAGE(char *msg);
+void NOTICE_MESSAGE(const char *msg);
 PG_LWGEOM *GEOS2POSTGIS(Geometry *geom, char want3d);
 Geometry * POSTGIS2GEOS(PG_LWGEOM *g);
 Geometry * LWGEOM2GEOS(LWGEOM *g);
@@ -117,11 +146,12 @@ LWPOLY *lwpoly_from_geometry(Geometry *g, char want3d);
 LWCOLLECTION *lwcollection_from_geometry(Geometry *geom, char want3d);
 LWGEOM *lwgeom_from_geometry(Geometry *g, char want3d);
 
-void NOTICE_MESSAGE(char *msg)
+void NOTICE_MESSAGE(const char *msg)
 {
 	elog(NOTICE,msg);
 }
 
+
 PG_FUNCTION_INFO_V1(postgis_geos_version);
 Datum postgis_geos_version(PG_FUNCTION_ARGS)
 {
@@ -135,16 +165,14 @@ Datum postgis_geos_version(PG_FUNCTION_ARGS)
 }
 
 
+
+#ifndef UNITE_USING_BUFFER
 /*
- * This is the final function for union/fastunite/geomunion
- * aggregate (still discussing the name). Will have
- * as input an array of Geometry pointers.
+ * This is the final function for GeomUnion
+ * aggregate. Will have as input an array of Geometries.
  * Will iteratively call GEOSUnion on the GEOS-converted
  * versions of them and return PGIS-converted version back.
  * Changing combination order *might* speed up performance.
- *
- * Geometries in the array are pfree'd as soon as possible.
- *
  */
 PG_FUNCTION_INFO_V1(unite_garray);
 Datum unite_garray(PG_FUNCTION_ARGS)
@@ -165,6 +193,8 @@ Datum unite_garray(PG_FUNCTION_ARGS)
 	call++;
 #endif
 
+	//lwnotice("GEOS incremental union");
+
 	datum = PG_GETARG_DATUM(0);
 
 	/* Null array, null geometry (should be empty?) */
@@ -199,7 +229,7 @@ Datum unite_garray(PG_FUNCTION_ARGS)
 #endif
 
 		// Check is3d flag
-		if ( TYPE_NDIMS(geom->type) > 2 ) is3d = 1;
+		is3d = TYPE_HASZ(geom->type);
 
 		// Check SRID homogeneity and initialize geos result
 		if ( ! i )
@@ -213,12 +243,7 @@ Datum unite_garray(PG_FUNCTION_ARGS)
 		}
 		else
 		{
-			if ( SRID != pglwgeom_getSRID(geom) )
-			{
-				elog(ERROR,
-					"Operation on mixed SRID geometries");
-				PG_RETURN_NULL();
-			}
+			errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom));
 		}
 		
 		g1 = POSTGIS2GEOS(pgis_geom);
@@ -255,6 +280,150 @@ Datum unite_garray(PG_FUNCTION_ARGS)
 
 }
 
+#else // def UNITE_USING_BUFFER
+
+/*
+ * This is the final function for GeomUnion
+ * aggregate. Will have as input an array of Geometries.
+ * Builds a GEOMETRYCOLLECTION from input and call
+ * GEOSBuffer(collection, 0) on the GEOS-converted
+ * versions of it. Returns PGIS-converted version back.
+ */
+PG_FUNCTION_INFO_V1(unite_garray);
+Datum unite_garray(PG_FUNCTION_ARGS)
+{
+	Datum datum;
+	ArrayType *array;
+	int is3d = 0;
+	int nelems, i, ngeoms, npoints;
+	PG_LWGEOM *result=NULL;
+	Geometry **geoms, *collection;
+	Geometry *g1, *geos_result=NULL;
+	int SRID=-1;
+	size_t offset;
+#ifdef PGIS_DEBUG
+	static int call=1;
+#endif
+
+#ifdef PGIS_DEBUG
+	call++;
+#endif
+
+	//lwnotice("GEOS buffer union");
+
+	datum = PG_GETARG_DATUM(0);
+
+	/* Null array, null geometry (should be empty?) */
+	if ( (Pointer *)datum == NULL ) PG_RETURN_NULL();
+
+	array = (ArrayType *) PG_DETOAST_DATUM(datum);
+
+	nelems = ArrayGetNItems(ARR_NDIM(array), ARR_DIMS(array));
+
+#ifdef PGIS_DEBUG
+	elog(NOTICE, "unite_garray: number of elements: %d", nelems);
+#endif
+
+	if ( nelems == 0 ) PG_RETURN_NULL();
+
+	/* One-element union is the element itself */
+	if ( nelems == 1 ) PG_RETURN_POINTER((PG_LWGEOM *)(ARR_DATA_PTR(array)));
+
+	geoms = lwalloc(sizeof(Geometry *)*nelems);
+
+	/* We need geos here */
+	initGEOS(MAXIMUM_ALIGNOF);
+
+	offset = 0; i=0;
+	ngeoms = 0; npoints=0;
+//lwnotice("Nelems %d, MAXGEOMSPOINST %d", nelems, MAXGEOMSPOINTS);
+	while (!result) 
+	{
+		PG_LWGEOM *geom = (PG_LWGEOM *)(ARR_DATA_PTR(array)+offset);
+		offset += INTALIGN(geom->size);
+
+		// Check is3d flag
+		is3d = TYPE_HASZ(geom->type);
+
+		// Check SRID homogeneity 
+		if ( ! i ) SRID = pglwgeom_getSRID(geom);
+		else errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom));
+
+		geoms[ngeoms] =
+			g1 = POSTGIS2GEOS(geom);
+			//lwgeom_deserialize(SERIALIZED_FORM(geom));
+
+		npoints += GEOSGetNumCoordinate(geoms[ngeoms]);
+
+		++ngeoms;
+		++i;
+
+//lwnotice("Loop %d, npoints: %d", i, npoints);
+
+		/*
+		 * Maximum count of geometry points reached
+		 * or end of them, collect and buffer(0).
+		 */
+		if ( (npoints>=MAXGEOMSPOINTS && ngeoms>1) || i==nelems)
+		{
+//lwnotice(" CHUNK (ngeoms:%d, npoints:%d, left:%d)", ngeoms, npoints, nelems-i);
+
+			//collection = (LWGEOM *)
+			//	lwcollection_construct(COLLECTIONTYPE,
+			//	-1, NULL, ngeoms, lwgeoms);
+			collection = PostGIS2GEOS_collection(COLLECTIONTYPE,
+				geoms, ngeoms, -1, is3d);
+
+			//g1 = LWGEOM2GEOS(collection);
+
+			//lwgeom_release(collection);
+
+			//geos_result = GEOSBuffer(g1, 0, 0);
+			geos_result = GEOSBuffer(collection, 0, 0);
+			if ( geos_result == NULL )
+			{
+				GEOSdeleteGeometry(g1);
+				lwerror("GEOS buffer() threw an error!");
+			}
+			//GEOSdeleteGeometry(g1);
+			GEOSdeleteGeometry(collection);
+
+//lwnotice(" Buffer() executed");
+
+			/*
+			 * If there are no other geoms in input
+			 * we've finished, otherwise we push
+			 * the result back on the input stack.
+			 */
+			if ( i == nelems )
+			{
+//lwnotice("  Final result points: %d", GEOSGetNumCoordinate(geos_result));
+				GEOSSetSRID(geos_result, SRID);
+				result = GEOS2POSTGIS(geos_result, is3d);
+				GEOSdeleteGeometry(geos_result);
+//lwnotice(" Result computed");
+			}
+			else
+			{
+				//lwgeoms = lwalloc(sizeof(LWGEOM *)*MAXGEOMS);
+				//lwgeoms[0] = lwgeom_from_geometry(geos_result, is3d);
+				geoms[0] = geos_result;
+				ngeoms=1;
+				npoints = GEOSGetNumCoordinate(geoms[0]);
+//lwnotice("  Result pushed back on lwgeoms array (npoints:%d)", npoints);
+			}
+		}
+	}
+
+
+	//compressType(result);
+
+	PG_RETURN_POINTER(result);
+
+}
+
+#endif // def UNITE_USING_BUFFER
+
 
 //select geomunion('POLYGON((0 0, 10 0, 10 10, 0 10, 0 0))','POLYGON((5 5, 15 5, 15 7, 5 7, 5 5))');
 PG_FUNCTION_INFO_V1(geomunion);
@@ -274,15 +443,11 @@ Datum geomunion(PG_FUNCTION_ARGS)
 	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
 	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
-	is3d = ( TYPE_NDIMS(geom1->type) > 2 ) ||
-		( TYPE_NDIMS(geom2->type) > 2 );
+	is3d = ( TYPE_HASZ(geom1->type) ) ||
+		( TYPE_HASZ(geom2->type));
 
 	SRID = pglwgeom_getSRID(geom1);
-	if ( SRID != pglwgeom_getSRID(geom2) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom2));
 
 	initGEOS(MAXIMUM_ALIGNOF);
 //elog(NOTICE,"in geomunion");
@@ -376,15 +541,10 @@ Datum symdifference(PG_FUNCTION_ARGS)
 	geom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
-	is3d = ( TYPE_NDIMS(geom1->type) > 2 ) ||
-		( TYPE_NDIMS(geom2->type) > 2 );
+	is3d = ( TYPE_HASZ(geom1->type) ) || ( TYPE_HASZ(geom2->type) );
 
 	SRID = pglwgeom_getSRID(geom1);
-	if ( SRID != pglwgeom_getSRID(geom2) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom2));
 
 	initGEOS(MAXIMUM_ALIGNOF);
 
@@ -506,7 +666,7 @@ Datum boundary(PG_FUNCTION_ARGS)
 #ifdef PROFILE
 	profstart(PROF_G2P);
 #endif
-	result = GEOS2POSTGIS(g3, TYPE_NDIMS(geom1->type) > 2);
+	result = GEOS2POSTGIS(g3, TYPE_HASZ(geom1->type));
 #ifdef PROFILE
 	profstart(PROF_P2G1);
 #endif
@@ -586,7 +746,7 @@ Datum convexhull(PG_FUNCTION_ARGS)
 #ifdef PROFILE
 	profstart(PROF_G2P);
 #endif
-	lwout = lwgeom_from_geometry(g3, TYPE_NDIMS(geom1->type) > 2);
+	lwout = lwgeom_from_geometry(g3, TYPE_HASZ(geom1->type));
 #ifdef PROFILE
 	profstop(PROF_G2P);
 #endif
@@ -601,9 +761,7 @@ Datum convexhull(PG_FUNCTION_ARGS)
 	/* Copy input bbox if any */
 	if ( getbox2d_p(SERIALIZED_FORM(geom1), &bbox) )
 	{
-		lwout->bbox = &bbox;
-		/* Mark lwgeom bbox to be externally owned */
-		TYPE_SETHASBBOX(lwout->type, 1);
+		lwout->bbox = box2d_clone(&bbox); 
 	}
 
 	result = pglwgeom_serialize(lwout);
@@ -682,7 +840,7 @@ Datum buffer(PG_FUNCTION_ARGS)
 #ifdef PROFILE
 	profstart(PROF_G2P);
 #endif
-	result = GEOS2POSTGIS(g3, TYPE_NDIMS(geom1->type) > 2);
+	result = GEOS2POSTGIS(g3, TYPE_HASZ(geom1->type));
 #ifdef PROFILE
 	profstop(PROF_G2P);
 #endif
@@ -726,15 +884,10 @@ Datum intersection(PG_FUNCTION_ARGS)
 	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
 	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
-	is3d = ( TYPE_NDIMS(geom1->type) > 2 ) ||
-		( TYPE_NDIMS(geom2->type) > 2 );
+	is3d = ( TYPE_HASZ(geom1->type) ) || ( TYPE_HASZ(geom2->type) );
 
 	SRID = pglwgeom_getSRID(geom1);
-	if ( SRID != pglwgeom_getSRID(geom2) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom2));
 
 	initGEOS(MAXIMUM_ALIGNOF);
 
@@ -846,15 +999,10 @@ Datum difference(PG_FUNCTION_ARGS)
 	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
 	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
-	is3d = ( TYPE_NDIMS(geom1->type) > 2 ) ||
-		( TYPE_NDIMS(geom2->type) > 2 );
+	is3d = ( TYPE_HASZ(geom1->type) ) || ( TYPE_HASZ(geom2->type) );
 
 	SRID = pglwgeom_getSRID(geom1);
-	if ( SRID != pglwgeom_getSRID(geom2) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom2));
 
 	initGEOS(MAXIMUM_ALIGNOF);
 
@@ -972,7 +1120,7 @@ Datum pointonsurface(PG_FUNCTION_ARGS)
 #ifdef PROFILE
 	profstart(PROF_G2P);
 #endif
-	result = GEOS2POSTGIS(g3, (TYPE_NDIMS(geom1->type) > 2));
+	result = GEOS2POSTGIS(g3, (TYPE_HASZ(geom1->type)));
 #ifdef PROFILE
 	profstop(PROF_G2P);
 #endif
@@ -1046,7 +1194,7 @@ Datum centroid(PG_FUNCTION_ARGS)
 #ifdef PROFILE
 	profstart(PROF_G2P);
 #endif
-	result = GEOS2POSTGIS(geosresult, (TYPE_NDIMS(geom->type) > 2));
+	result = GEOS2POSTGIS(geosresult, (TYPE_HASZ(geom->type)));
 #ifdef PROFILE
 	profstop(PROF_G2P);
 #endif
@@ -1168,6 +1316,7 @@ Datum overlaps(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not overlap
@@ -1246,6 +1395,7 @@ Datum contains(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box is not completely inside
@@ -1324,6 +1474,7 @@ Datum within(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom1 bounding box is not completely inside
@@ -1403,6 +1554,7 @@ Datum crosses(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not overlap
@@ -1482,6 +1634,7 @@ Datum intersects(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not overlap
@@ -1558,6 +1711,7 @@ Datum touches(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not overlap
@@ -1636,6 +1790,7 @@ Datum disjoint(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not overlap
@@ -1713,6 +1868,7 @@ Datum relate_pattern(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	initGEOS(MAXIMUM_ALIGNOF);
 
@@ -1784,6 +1940,7 @@ Datum relate_full(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 
 	initGEOS(MAXIMUM_ALIGNOF);
@@ -1879,6 +2036,7 @@ Datum geomequals(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not equal
@@ -2354,7 +2512,7 @@ LWGEOM2GEOS(LWGEOM *lwgeom)
 			}
 			return PostGIS2GEOS_collection(TYPE_GETTYPE(col->type),
 				collected, col->ngeoms, col->SRID,
-				TYPE_NDIMS(col->type)>2);
+				TYPE_HASZ(col->type));
 
 		default:
 			lwerror("Unknown geometry type: %d",
@@ -2405,7 +2563,7 @@ Datum GEOSnoop(PG_FUNCTION_ARGS)
 	profstop(PROF_GRUN);
 #endif
 
-	result = GEOS2POSTGIS(geosgeom, TYPE_NDIMS(geom->type) > 2);
+	result = GEOS2POSTGIS(geosgeom, TYPE_HASZ(geom->type));
 	GEOSdeleteGeometry(geosgeom);
 
 #ifdef PGIS_DEBUG_CONVERTER
@@ -2417,8 +2575,8 @@ Datum GEOSnoop(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(result);
 }
 
-PG_FUNCTION_INFO_V1(GEOS_polygonize_garray);
-Datum GEOS_polygonize_garray(PG_FUNCTION_ARGS)
+PG_FUNCTION_INFO_V1(polygonize_garray);
+Datum polygonize_garray(PG_FUNCTION_ARGS)
 {
 	Datum datum;
 	ArrayType *array;
@@ -2447,7 +2605,7 @@ Datum GEOS_polygonize_garray(PG_FUNCTION_ARGS)
 	nelems = ArrayGetNItems(ARR_NDIM(array), ARR_DIMS(array));
 
 #ifdef PGIS_DEBUG
-	elog(NOTICE, "GEOS_polygonize_garray: number of elements: %d", nelems);
+	elog(NOTICE, "polygonize_garray: number of elements: %d", nelems);
 #endif
 
 	if ( nelems == 0 ) PG_RETURN_NULL();
@@ -2469,23 +2627,21 @@ Datum GEOS_polygonize_garray(PG_FUNCTION_ARGS)
 		}
 		else
 		{
-			if ( SRID != pglwgeom_getSRID(geom) )
-			{
-	elog(ERROR, "polygonize: operation on mixed SRID geometries");
-	PG_RETURN_NULL();
-			}
+			errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom));
 		}
 	}
 
 #ifdef PGIS_DEBUG
-	elog(NOTICE, "GEOS_polygonize_garray: invoking GEOSpolygonize");
+	elog(NOTICE, "polygonize_garray: invoking GEOSpolygonize");
 #endif
 
 	geos_result = GEOSpolygonize(vgeoms, nelems);
 #ifdef PGIS_DEBUG
-	elog(NOTICE, "GEOS_polygonize_garray: GEOSpolygonize returned");
+	elog(NOTICE, "polygonize_garray: GEOSpolygonize returned");
 #endif
-	//pfree(vgeoms);
+	for (i=0; i<nelems; ++i) GEOSdeleteGeometry(vgeoms[i]);
+	pfree(vgeoms);
+
 	if ( ! geos_result ) PG_RETURN_NULL();
 
 	GEOSSetSRID(geos_result, SRID);
@@ -2503,10 +2659,240 @@ Datum GEOS_polygonize_garray(PG_FUNCTION_ARGS)
 
 }
 
+/*
+ * Take a geometry and return an areal geometry
+ * (Polygon or MultiPolygon).
+ * Actually a wrapper around GEOSpolygonize, 
+ * transforming the resulting collection into
+ * a valid polygonzl Geometry.
+ */
+PG_FUNCTION_INFO_V1(LWGEOM_buildarea);
+Datum LWGEOM_buildarea(PG_FUNCTION_ARGS)
+{
+	int is3d = 0;
+	unsigned int i, ngeoms;
+	PG_LWGEOM *result;
+	LWGEOM *lwg;
+	Geometry *geos_result, *shp;
+	Geometry *vgeoms[1];
+	int SRID=-1;
+#ifdef PGIS_DEBUG
+	static int call=1;
+#endif
+
+#ifdef PGIS_DEBUG
+	call++;
+#endif
+
+	//lwnotice("buildarea called");
+
+	PG_LWGEOM *geom = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	SRID = pglwgeom_getSRID(geom);
+	is3d = (TYPE_HASZ(geom->type));
+
+#ifdef PGIS_DEBUG
+	lwnotice("LWGEOM_buildarea got geom @ %x", geom);
+#endif
+
+	initGEOS(MAXIMUM_ALIGNOF);
+
+	vgeoms[0] = POSTGIS2GEOS(geom);
+	geos_result = GEOSpolygonize(vgeoms, 1);
+	GEOSdeleteGeometry(vgeoms[0]);
+
+#ifdef PGIS_DEBUG
+	lwnotice("GEOSpolygonize returned @ %x", geos_result);
+#endif
+
+	/* Null return from GEOSpolygonize */
+	if ( ! geos_result ) PG_RETURN_NULL();
+
+	/*
+	 * We should now have a collection
+	 */
+#if PARANOIA_LEVEL > 0
+	if ( GEOSGeometryTypeId(geos_result) != COLLECTIONTYPE )
+	{
+		GEOSdeleteGeometry(geos_result);
+		lwerror("Unexpected return from GEOSpolygonize");
+		PG_RETURN_NULL();
+	}
+#endif
+
+	ngeoms = GEOSGetNumGeometries(geos_result);
+
+#ifdef PGIS_DEBUG
+	lwnotice("GEOSpolygonize: ngeoms in polygonize output: %d", ngeoms);
+#endif
+
+	/*
+	 * No geometries in collection, return NULL
+	 */
+	if ( ngeoms == 0 ) 
+	{
+		GEOSdeleteGeometry(geos_result);
+		PG_RETURN_NULL();
+	}
+
+	/*
+	 * Return first geometry if we only have one in collection,
+	 * to avoid the unnecessary Geometry clone below.
+	 */
+	if ( ngeoms == 1 )
+	{
+		shp = GEOSGetGeometryN(geos_result, 0);
+		lwg = lwgeom_from_geometry(shp, is3d);
+		lwg->SRID = SRID;
+		result = pglwgeom_serialize(lwg);
+		lwgeom_release(lwg);
+		GEOSdeleteGeometry(geos_result);
+		PG_RETURN_POINTER(result);
+	}
+
+#if PGIS_DEBUG
+	lwnotice("Returned geom has %d elems");
+#endif
+
+	/* 
+	 * Iteratively invoke symdifference on outer rings
+	 * as suggested by Carl Anderson:
+	 * postgis-devel/2005-December/001805.html
+	 */
+	shp = NULL;
+	for (i=0; i<ngeoms; ++i)
+	{
+		Geometry *extring, *tmp;
+
+		/*
+		 * Construct a Polygon from geometry i exterior ring
+		 */
+		extring = GEOSCreatePolygon(
+			GEOSCloneGeometry(GEOSGetExteriorRing(
+				GEOSGetGeometryN(geos_result, i))),
+			NULL, 0);
+
+		if ( extring == NULL ) // exception
+		{
+			lwerror("GEOSCreatePolygon threw an exception");
+			PG_RETURN_NULL();
+		}
+
+		if ( shp == NULL )
+		{
+			shp = extring;
+		}
+		else
+		{
+			tmp = GEOSSymDifference(shp, extring);
+			GEOSdeleteGeometry(shp);
+			GEOSdeleteGeometry(extring);
+			shp = tmp;
+		}
+	}
+
+	GEOSdeleteGeometry(geos_result);
+
+	GEOSSetSRID(shp, SRID);
+	result = GEOS2POSTGIS(shp, is3d);
+	GEOSdeleteGeometry(shp);
+
+#if PARANOIA_LEVEL > 0
+	if ( result == NULL )
+	{
+		lwerror("serialization error");
+		PG_RETURN_NULL(); //never get here
+	}
+
+#endif
+
+	PG_RETURN_POINTER(result);
+
+}
+
+PG_FUNCTION_INFO_V1(linemerge);
+Datum linemerge(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM	*geom1;
+	Geometry *g1,*g3;
+	PG_LWGEOM *result;
+
+#ifdef PROFILE
+	profstart(PROF_QRUN);
+#endif
+
+	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+
+	initGEOS(MAXIMUM_ALIGNOF);
+
+#ifdef PROFILE
+	profstart(PROF_P2G1);
+#endif
+	g1 = POSTGIS2GEOS(geom1);
+#ifdef PROFILE
+	profstop(PROF_P2G1);
+#endif
+
+#ifdef PROFILE
+	profstart(PROF_GRUN);
+#endif
+	g3 = GEOSLineMerge(g1);
+#ifdef PROFILE
+	profstop(PROF_GRUN);
+#endif
+
+	if (g3 == NULL)
+	{
+		elog(ERROR,"GEOS LineMerge() threw an error!");
+		GEOSdeleteGeometry(g1);
+		PG_RETURN_NULL(); //never get here
+	}
+
+
+//	elog(NOTICE,"result: %s", GEOSasText(g3) ) ;
+
+	GEOSSetSRID(g3, pglwgeom_getSRID(geom1));
+
+#ifdef PROFILE
+	profstart(PROF_G2P);
+#endif
+	result = GEOS2POSTGIS(g3, TYPE_HASZ(geom1->type));
+#ifdef PROFILE
+	profstop(PROF_G2P);
+#endif
+	if (result == NULL)
+	{
+		GEOSdeleteGeometry(g1);
+		GEOSdeleteGeometry(g3);
+		elog(ERROR,"GEOS LineMerge() threw an error (result postgis geometry formation)!");
+		PG_RETURN_NULL(); //never get here
+	}
+	GEOSdeleteGeometry(g1);
+	GEOSdeleteGeometry(g3);
+
+
+	//compressType(result);  // convert multi* to single item if appropriate
+
+#ifdef PROFILE
+	profstop(PROF_QRUN);
+	profreport("geos",geom1, NULL, result);
+#endif
+
+	PG_FREE_IF_COPY(geom1, 0);
+
+	PG_RETURN_POINTER(result);
+}
+
 Datum JTSnoop(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(JTSnoop);
 Datum JTSnoop(PG_FUNCTION_ARGS)
 {
-	elog(NOTICE, "JTS support is disabled, use GEOSnoop instead");
+	elog(ERROR, "JTS support is disabled");
 	PG_RETURN_NULL();
 }
+
+PG_FUNCTION_INFO_V1(postgis_jts_version);
+Datum postgis_jts_version(PG_FUNCTION_ARGS)
+{
+	PG_RETURN_NULL();
+}
+
