@@ -6,7 +6,7 @@
 
 #pragma GCC java_exceptions
 
-#include "jts.h"
+#include <jts.h>
 #include <java/lang/String.h>
 #include <java/lang/System.h>
 #include <java/io/PrintStream.h>
@@ -16,6 +16,7 @@
 
 using namespace com::vividsolutions::jts::geom;
 using namespace com::vividsolutions::jts::io;
+using namespace com::vividsolutions::jts;
 using namespace java::lang;
 using namespace std;
 
@@ -105,7 +106,8 @@ extern "C" char JTSrelateCrosses(Geometry *g1, Geometry*g2);
 extern "C" char JTSrelateWithin(Geometry *g1, Geometry*g2);
 extern "C" char JTSrelateContains(Geometry *g1, Geometry*g2);
 extern "C" char JTSrelateOverlaps(Geometry *g1, Geometry*g2);
-extern "C" Geometry *JTSpolygonize(Geometry **geoms, unsigned int ngeoms);
+extern "C" Geometry *JTSPolygonize(Geometry **geoms, unsigned int ngeoms);
+extern "C" Geometry *JTSLineMerge(Geometry *geoms);
 extern "C" char *JTSversion();
 extern "C" char *JTSjtsport();
 extern "C" int JTSGeometryTypeId(Geometry *g1);
@@ -166,9 +168,10 @@ initJTS(noticefunc nf)
 	{
 		JvCreateJavaVM(NULL);
 		JvAttachCurrentThread(NULL, NULL);	
-		//JvInitClass(&Geometry::class$);
-		//JvInitClass(&GeometryFactory::class$);
-		//JvInitClass(&Coordinate::class$);
+		JvInitClass(&Geometry::class$);
+		JvInitClass(&GeometryFactory::class$);
+		JvInitClass(&Coordinate::class$);
+		JvInitClass(&JTSVersion::class$);
 
 		// NOTE: SRID will have to be changed after geometry creation
 		jtsGeomFactory = new GeometryFactory( new PrecisionModel(), -1);
@@ -735,9 +738,16 @@ JTSGetInteriorRingN(Geometry *g1,int n)
 }
 
 Geometry *
-JTSpolygonize(Geometry **g, unsigned int ngeoms)
+JTSPolygonize(Geometry **g, unsigned int ngeoms)
 {
-	NOTICE_MESSAGE("JTS polygonize unimplemented");
+	NOTICE_MESSAGE("JTS Polygonize unimplemented");
+	return NULL;
+}
+
+Geometry *
+JTSLineMerge(Geometry *g)
+{
+	NOTICE_MESSAGE("JTS LineMerge unimplemented");
 	return NULL;
 }
 
@@ -768,8 +778,11 @@ JTSGetSRID(Geometry *g1)
 char *
 JTSversion()
 {
-	char *res = strdup("unknown");
-	return res;
+	//char *res = strdup("unknown");
+        JTSVersion *v = JTSVersion::CURRENT_VERSION;
+        if ( ! v ) return "UNDEFINED";
+        return StringToChar(v->toString());
+	//return res;
 }
 
 bool 
