@@ -1,3 +1,15 @@
+/**********************************************************************
+ * $Id: lwline.c 2369 2006-05-30 08:38:58Z strk $
+ *
+ * PostGIS - Spatial Types for PostgreSQL
+ * http://postgis.refractions.net
+ * Copyright 2001-2006 Refractions Research Inc.
+ *
+ * This is free software; you can redistribute and/or modify it under
+ * the terms of the GNU General Public Licence. See the COPYING file.
+ * 
+ **********************************************************************/
+
 /* basic LWLINE functions */
 
 #include <stdio.h>
@@ -50,7 +62,7 @@ lwline_deserialize(uchar *serialized_form)
 
 	if ( lwgeom_getType(type) != LINETYPE)
 	{
-		lwerror("lwline_deserialize: attempt to deserialize a line when its not really a line");
+		lwerror("lwline_deserialize: attempt to deserialize a line which is really a %s", lwgeom_typename(type));
 		return NULL;
 	}
 
